/*
TITLE: XREF - Cross Reference;
VERSION: 1.0;
DATE: 07/31/1986;
DESCRIPTION:
 "Cross reference text from stdin to stdout.
 Optimized name storage allocation.";
KEYWORDS: binary tree, name, reference, symbol, xref;
SYSTEM: CP/M, any;
FILENAME: XREF.C;
WARNINGS: "Assumes 16-bit pointers.";
SEE-ALSO: none;
AUTHORS: Mark Antony Washburn;
COMPILERS: Small-C, any C compiler; 
REFERENCES:
 AUTHORS: Nickolas Wirth;
 TITLE: "Algorithms+Data Structures=Programs";
 CITATION: "p.206"
ENDREF;
*/

#include <A:STDIO.H>  /* find STDIO.H */

#define MEMOVERHEAD (1024)  /* leave room for stack */

#define MAXWORDLEN 79  /* maximum name length - 1 */
#define MAXWORDBUF 80  /* maximum name length */ 
#define NUMPERLINE 14  /* assumes 80 columns */
#define MAXLINENUM 9999  /* maximum number of source lines */

#define DNLEFT 0  /* name node structure definitions */ 
#define DNRIGHT 2
#define DNFIRST 4
#define DNLAST 6
#define DNID 8

#define DRLINE 0  /* reference link list structure definitions */
#define DRNEXT 2
#define DRLN 4

char *membtm,*memtop;  /* pointers to bottom and top of memory */

char *root;  /* root node pointer */
char *nword;  /* global name node pointer */ 
/* left and right node pointers, first and last reference links */
int *nleft,*nright,*nfirst,*nlast;
int *rline,*rnext;  /* global pointers reference instance */

int c;  /* current char */
int k;  /* current word index */
int n;  /* current line number */
char id[MAXWORDBUF];  /* current word */

/*
**  ouput character to stdout, then fetch next character
**  from stdin
*/
outcinc() {
  putchar(c);
  c = getchar(c);
  }

/*
** output a string to stdout
*/
outstr(l) char *l; {
  fputs(l, stdout);
  }

/*
**  output a right justified integer to stdout
*/
outdec(i) int i; {
  char l[5];
  itod(i, l, 5);
  fputs(l, stdout);
  }

/*
**  get as much memory as possible for symbol tree
*/
initmem() {
  int max;
  max = avail(YES);
  max -= MEMOVERHEAD;
  membtm = malloc(max);
  memtop = membtm + max;
  }

/* 
**  get memory for next entry, otherwise abort
*/
getmem(i) int i; {
  char *memp;
  if(memtop > membtm + i)  {
    memp = membtm;
    membtm += i;
    return(memp);
    }
  else {
    fputs("Not enough memory", stderr);
    abort(2);
    }
  }

/*
**  assign node pointers
*/
nodeptrs(w) char *w; {
  nleft = w;
  nright = w + DNRIGHT;
  nfirst = w + DNFIRST;
  nlast = w + DNLAST;
  nword = w + DNID;
  }

/*
**  assign reference pointers
*/
refptrs(x) char *x; {
  rline = x;
  rnext = x + DRNEXT;
  }

/*
**  if id is a new node then insert word and reference 
**  otherwise insert next reference
*/
insert(w) char *w; {
  int *x, *lnode;
  int i;
  if(w == NULL) {
    w = getmem(DNID + k);
    nodeptrs(w);
    strcpy(nword, id);
    *nleft = NULL;
    *nright = NULL;
    refptrs(getmem(DRLN));
    *rline = n;
    *rnext = NULL;
    *nfirst = rline;
    *nlast = rline;
    if(root == NULL) root = w;
    else return(w);
    }
  else {
    nodeptrs(w);
    i = strcmp(id, nword);
    if(i == 0) {
      x = getmem(DRLN);
      refptrs(x);
      *rline = n;
      *rnext =) End of article 4933 (of 4956)--what next? [npq] 

Article 4934 (22 more) in net.lang.c:
From: cottrell@nbs-vms.ARPA (COTTRELL, JAMES)
Subject: This Sentence is False
Message-ID: <1118@brl-tgr.ARPA>
Date: 30 Aug 85 01:20:41 GMT
Date-Received: 2 Sep 85 06:25:09 GMT
Sender: news@brl-tgr.ARPA
Lines: 97

--MORE--(12%)/* Fiercely I go into battle:
> I have always thought that to be a machine dependancy (the value of true and
> false).  Maybe I'm wrong.  But, different machines DO have different ideas
> of which is true and false (at the assembler level).  And it is simply
> a convention.

Machines have no concept of truth. They can determine if a condition
(equality, greater than, negative, overflow, etc.) exists and then
*dynamically* alter the course of their control flow. But no machine
I know of has a `branch if true' instruxion. Note I am not talking
about the abstraxion a la 68k where an unconditional branch is named
`branch if true' & the converse is called `branch if false'. I am 
talking about a *conditional* `branch if true'.
 
> Still though, #define TRUE (1==1) is very obvious, to the point, correct,
> proper, and all sorts of things.  And it doesn't require one to know
> that detail about C that the convention is ~0 == TRUE and 0 == FALSE.

Your attention span must be even shorter than mine. (What were we talking
about :-) If you are consistant, ignorance is bliss. But what about
fred's code ov